import os

from django.conf import settings
from django.core import exceptions
from django.utils.importlib import import_module
from django_nose.plugin import DjangoSetUpPlugin, ResultPlugin, TestReorderer
from django_nose.runner import NoseTestSuiteRunner

import nose.core

from plugin import RedRoverFilter


def _get_plugins_from_settings():
  plugins = (list(getattr(settings, 'NOSE_PLUGINS', [])) +
             ['django_nose.plugin.TestReorderer',
              'redrover.plugin.RedRoverFilter'])

  for plug_path in plugins:
    try:
      dot = plug_path.rindex('.')
    except ValueError:
      raise exceptions.ImproperlyConfigured(
        "%s isn't a Nose plugin module" % plug_path)
    p_mod, p_classname = plug_path[:dot], plug_path[dot + 1:]

    try:
      mod = import_module(p_mod)
    except ImportError, e:
      raise exceptions.ImproperlyConfigured(
        'Error importing Nose plugin module %s: "%s"' % (p_mod, e))

    try:
      p_class = getattr(mod, p_classname)
    except AttributeError:
      raise exceptions.ImproperlyConfigured(
        'Nose plugin module "%s" does not define a "%s"' %
        (p_mod, p_classname))

    yield p_class()


class RedRoverRunner(NoseTestSuiteRunner):

  def __init__(self, *args, **kwargs):
    if 'NOSE_REDROVER' not in os.environ:
      os.environ['NOSE_REDROVER'] = '1'
    super(RedRoverRunner, self).__init__(*args, **kwargs)

  def run_suite(self, nose_argv):
    result_plugin = ResultPlugin()
    plugins_to_add = [DjangoSetUpPlugin(self),
                      result_plugin,
                      TestReorderer(),
                      RedRoverFilter()]

    for plugin in _get_plugins_from_settings():
        plugins_to_add.append(plugin)

    nose.core.TestProgram(argv=nose_argv, exit=False,
                          addplugins=plugins_to_add)
    return result_plugin.result



class BaseAssertion(object):
  passes = None

  @property
  def message(self):
    """Should be implemented by child classes."""
    raise NotImplementedError


class EqualAssertion(BaseAssertion):
  """Compare the equivalence of two objects."""

  def __init__(self, subject, other):
    self.subject = subject
    self.other = other
    self.passes = subject == other

  @property
  def message(self):
    if self.passes:
      msg = '{subject_type} {subject_value} equals {other}'
    else:
      msg = '{subject_type} {subject_value} does not equal {other}'
    return msg.format(
      subject_type=type(self.subject).__name__.capitalize(),
      subject_value=str(self.subject),
      other=str(self.other))



from types import FunctionType

from django.test import TestCase


def do_decorate(attr, value):
  return (
    '__' not in attr and
    'setUp' is not attr and
    'setUpClass' is not attr and
    'tearDown' is not attr and
    'tearDownClass' is not attr and
    'run' is not attr and
    'skipTest' is not attr and
    'debug' is not attr and
    isinstance(value, FunctionType))


def decorate_all():

  class DecorateAll(type):

    def __new__(cls, name, bases, dct):
      for attr, value in dct.iteritems():
        if do_decorate(attr, value):
          value.__name__ = 'test_%s' % value.__name__
          value.__globals__.update({
            'it': Subject(dct['subject'])})
      return super(DecorateAll, cls).__new__(
        cls, name, bases, dct)

    def __setattr__(self, attr, value):
      if do_decorate(attr, value):
        value.__name__ = 'test_%s' % value.__name__
      super(DecorateAll, self).__setattr__(attr, value)

  return DecorateAll


class Subject(object):

  def __init__(self, subject):
    self.subject = subject

  def should(self, cls, *args, **kwargs):
    assertion = cls(self.get_subject, *args, **kwargs)
    if not assertion.passes:
      raise AssertionError(assertion.message)
    return True

  def should_not(self, cls, *args, **kwargs):
    assertion = cls(self.get_subject, *args, **kwargs)
    if assertion.passes:
      raise AssertionError(assertion.message)
    return True

  @property
  def get_subject(self):
    return self.subject


class RedRover(TestCase):

  __metaclass__ = decorate_all()



import os

import termstyle
from rednose import RedNose


class RedRoverFilter(RedNose):
  """
  Suppress the last traceback entry since that will always be raised by
  redrover.

  """
  env_opt = 'NOSE_REDROVER'
  env_opt_color = 'NOSE_REDROVER_COLOR'
  score = 199

  def options(self, parser, env=os.environ):
    global REDNOSE_DEBUG
    rednose_on = bool(env.get(self.env_opt, False))
    rednose_color = env.get(self.env_opt_color, 'auto')
    REDNOSE_DEBUG = bool(env.get('REDNOSE_DEBUG', False))

    parser.add_option(
      "--redrover-rednose", action="store_true",
      default=rednose_on, dest="redrover_rednose",
      help="enable colour output (alternatively, set $%s=1)" % (self.env_opt,))
    parser.add_option(
      "--no-redrover-color", action="store_false",
      dest="redrover_rednose",
      help="disable colour output")
    parser.add_option(
      "--force-redrover-color", action="store_const",
      dest='redrover_rednose_color',
      default=rednose_color,
      const='force',
      help="force colour output when not using a TTY " +
        "(alternatively, set $%s=force)" % (self.env_opt_color,))
    parser.add_option(
      "--redrover-immediate", action="store_true",
      default=False,
      help="print errors and failures as they happen, as well as at the end")

  def configure(self, options, conf):
    if options.redrover_rednose:
      self.enabled = True
      termstyle_init = {
        'force': termstyle.enable,
        'off': termstyle.disable
      }.get(options.redrover_rednose_color, termstyle.auto)
      termstyle_init()

      self.immediate = options.immediate
      self.verbose = options.verbosity >= 2

  def _fmt_traceback(self, trace):
    """format a traceback"""
    ret = []
    ret.append(termstyle.default("   Traceback (most recent call last):"))
    current_trace = trace
    while current_trace is not None:
      line = self._file_line(current_trace)
      if line is not None and 'redrover/case.py' not in line:
        ret.append(line)
      current_trace = current_trace.tb_next
    return '\n'.join(ret)



#!/usr/bin/env python
"""
RedRover
========

RedRover is a testing utility suite for Django.  It wraps other powerful
tools such as Nose and Selinium into an easy to use and read syntax.

At its core, RedRover borrows from other python projects and provides
an interface with custom syntax sugar inspired by the RSpec and Capybara
projects that are used widely with Ruby on Rails.

:copyright: (c) 2012 by Dustin Farris
:license: BSD, see LICENSE for more details.

"""

from setuptools import setup, find_packages


# Hack to prevent stupid "TypeError: 'NoneType' object is not callable" error
# in multiprocessing/util.py _exit_function when running `python
# setup.py test` (see
# http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
for m in ('multiprocessing', 'billiard'):
    try:
        __import__(m)
    except ImportError:
        pass

tests_require = [
  'pytest',
  'pytest-django']

install_requires = [
  'Django>=1.4',
  'django_nose',
  'rednose==0.3.3',
  'splinter']

setup(
  name='redrover',
  version='0.2.0',
  author='Dustin Farris',
  author_email='dustin@dustinfarris.com',
  url='https://github.com/dustinfarris/redrover',
  description='A testing utility suite for Django.',
  long_description=__doc__,
  package_dir={'': 'src'},
  packages=find_packages('src'),
  zip_safe=False,
  install_requires=install_requires,
  tests_require=tests_require,
  test_suite='runtests.runtests',
  license='BSD',
  include_package_data=True,
  classifiers=[
    'Framework :: Django',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Operating System :: OS Independent',
    'Topic :: Software Development'])